\ Clean Songlist - for the Digital Piano -- include 3iPiano.fs  

\ TODO: ADD REST NOTATIONS
\ TODO: ADD variable tones (half and sixteenth notes)
\ TODO: FIX - leave doesn't work
\ Design: this plays by reading notes or directly e.g. 1 .. 2 .. 3 ..

: include-song-files
  s" 3iForth.fs"        included  500 ms
  s" 3iSound-4x4.fs"    included  500 ms
  s" 3iShift.fs"        included  500 ms
  s" 3iPianoLights.fs"  included  500 ms
;

\ include-song-files

\ MUSICAL NOTATION FOR SONGLIST.fs
\ Design: SONG GRAMMAR consists of (1) 8 diatonic notes 1..8,  (2) 6 chromatic notes 9-14, (3) octave down/up, (4) musical rest short/long
\ Design: the grammar allows writing music in easy-to-read 1-8 notation, #/% for sharp/flat, <</>> for octave down/up, and //,/// for short/long musical rest
\ Design: scaleroot sets default index to middle c  (c_ / 36 note or 262 Hz tone), span c_ to c'
\ Design: scaleroot gets adjusted to move virtually between octaves
variable scaleroot 
: scaleroot!      ( note-name -- ) scaleroot !    ;
: scaleroot@      ( -- note-name ) scaleroot @    ;
: reset-scaleroot ( -- )           c_  scaleroot! ;
reset-scaleroot

\ Design: 1 2 ... 8 notation for white keys indexed to middle c_
\ Design: Sharps and Flats added through decorating 1..8
\ Design: cmaj-map maps 1...8 (white keys) and 9..14 (black keys) to the sequential piano note index
9  constant 1#
9  constant 2%
10 constant 2#
10 constant 3%
11 constant 4#
11 constant 5%
12 constant 5#
12 constant 6%
13 constant 6#
13 constant 7%
14 constant 8#
\ map: diatonic to chromatic
create cmaj-map       0 c, 2 c, 4 c, 5 c, 7 c, 9 c, 11 c, 12 c, ( white keys ) 1 c, 3 c, 6 c, 8 c, 10 c, 13 c, ( black keys )
create note-to-0index 0 c, 2 c, 4 c, 5 c, 7 c, 9 c, 11 c, 12 c, -1 c, 1 c, 3 c, 6 c, 8 c, 10 c, 
: [...]  ( playlist# -- 0index# )     1 - cmaj-map + c@ ;  \ transforms playlist notation to sequential notes 0-indexed
: translatenote ( recordnote -- 0indexnote )  \ takes songlist grammar note [diatonic] (1..14) and converts to piano sequence (0..13) [chromatic]
    1 - bytes note-to-0index + c@ ;
\ : test 15 1 do i dup . translatenote dup . _soundlight 200 ms cr loop ;
\ map (inverse): chromatic to diatonic
create chr2cmaj-map  1 c, 9 c, 2 c, 10 c, 3 c, 4 c, 11 c, 5 c, 12 c, 6 c, 13 c, 7 c, 8 c, 14 c,
: chr2dia ( 0indexnote -- grammarnote )  \ takes 0index chromatic note and converts to 1-index cmaj note
    bytes chr2cmaj-map + c@ ;
\ : test 13 0 do i dup . chr2dia . cr loop ;

\  Design: on 32-key F__ to C'' keyboard seq# is -7 to 24  (piano-led board)
: [,,,]  ( 1-8# or 1-14# -- 0index# ) 1 - scale + c@ c_ - ;   \ provides relative index +/- of note referenced to c_ middle c for transposition

\ Design: non-note grammar are above the c.96 notes in the piano scale (extended manually to custom frequencies)
\ Design: virtual (software only) octave adjustments.  See ><< and >>> in 3iSound-4x4.fs for physical/hardware octave adjustments in buttons
$FC constant <<  \ octave down
$FD constant >>  \ octave up
: [<<] ( -- ) scaleroot@ 12 - scaleroot! ;   \ lower octave
: [>>] ( -- ) scaleroot@ 12 + scaleroot! ;   \ raise octave
: adjust-octave ( g:<<,>> --  ) >> - if [<<] else [>>] then ;

\ Design: musical rests
$FE constant //  \ short rest R1
$FF constant /// \ long  rest R2
: musical-rest ( g --  ) /// - if R1 else R2 then ;


\ Design: storage for SONG GRAMMAR g for playback, transpositions etc.
create songmem 512 bytes allot
variable songptr
variable songlen
: init-songptr    ( -- )   songmem songptr ! ;
: reset-songlen   ( -- )   0 songlen ! ;
: songlen++       ( -- )   songlen @ 1 + songlen ! ;
: songc!+         ( g -- ) songptr @   c!+  songptr ! songlen++ ;
: song-mem2list   ( n -- list n )  \ memory-array to list-stack
    >r songmem r@ 0 do c@+ swap loop drop r> ;
: showsongmem     ( n -- ) songmem over 0 do c@+ . loop drop ;  \ dump-memory
: showsonglist    ( n -- ) dup 0 do dup i - roll . loop drop ;  \ dump-list
init-songptr

\ OBSERVATION:  5flats  lowered 1 semitone gives a nice cmaj-white keys scale!
   

: pausedur 500 ms ;

: _key              ( -- 0/1 )     key esc-key = if leave then ;  \ BUG!!! esc-key needs to trigger leaving outerloop
: _sound            ( note# -- )  note ;            \ transform to note# (3iSound-4x4.fs) and plays note
: [light]           36 - 8 + ;
: _light            ( note# -- )  [light] kk ;     \ transform to light# (3iPianoLights.fs) and turns on light
: _soundlight       ( note# -- )  dup _light _sound ;
: _soundlightkey    ( note# -- )  dup . _soundlight _key ; \ advance on keypress, ESC to stop
: _soundlightbutton ( note# -- )  
    begin dup _soundlight poll-note-data translatenote scaleroot@ + 2dup = 
        if drop pausedur exitloop 
        else drop 32-clear pausedur pausedur continueloop 
        then 
    until drop ;
: _lightkey         ( 0index# -- )  _light _key ;
: _soundkey         ( 0index# -- )  _sound _key ;

variable playmode
0 constant  soundonly
1 constant  lightonly
2 constant  soundlight
3 constant  soundlightkey
4 constant  lightkey
5 constant  soundkey
6 constant  soundlightbutton
: playmode!  ( choice -- ) playmode ! ;
: soundonly-mode  soundonly playmode! ;
: soundlight-mode soundlight playmode! ;
: soundlightkey-mode soundlightkey playmode! ;
: soundlightbutton-mode soundlightbutton playmode! ;
: soundkey-mode soundkey playmode! ;
: lightkey-mode lightkey playmode! ;
soundonly-mode  \ default at startup

: playmode@  ( -- choice ) playmode @ ;
create playmode-map   ' _sound , ' _light , ' _soundlight , ' _soundlightkey , ' _lightkey , ' _soundkey , ' _soundlightbutton ,
  \ Design: .. doesn't always sound a note, if << or >> then it adjusts the octave and stops as the next note needs to be rolled in from the stack

\ CODE REVIEW: Get to a musical data structure -- 3 arrays
\ play directly from stack, left to right


\ ================= ALGORITHM -- EXTRACT / DISPLAY KEY SIGNATURE FROM SONG
\ \ Design: extract uniques
create songuniq 45 bytes allot \ enough to surround the 32-key pianolights board
variable uniqptr
variable uniqlen
: reset-uniqptr  songuniq uniqptr !   0 uniqlen ! ;
: uniqptr@   uniqptr @ ; 
: uniqptr++  uniqptr@ 1 + uniqptr ! ;
: uniqlen@  uniqlen @ ;
: uniqlen++  uniqlen @ 1 + uniqlen ! ;
: add-uniq ( x -- ) uniqptr@ ! uniqptr++ uniqlen++  ;  \ ignore non-note grammars
: show-uniq ( -- ) uniqlen@ 0 > if songuniq uniqlen@ 0 do c@+ . loop drop else ." empty " then ;

: end?  ( uptr -- 1/0 ) dup uniqptr@ = ;
: uniq? ( x uptr -- x uptr' 1/0 ) over swap c@+ rot = ;  \ CODE REVIEW: this is actually 0 for uniq, 1 for not uniq --- misnamed
: search-uniq ( x uptr -- 0/1 )     
    begin uniq?
        if  \ not-unique, i.e. matching entry found
            drop exitloop
        else \ uniq, i.e. entry not found, continue search unless at end of list
            end? 
                if \ at end of list
                    swap add-uniq exitloop 
                else continueloop 
                then 
        then 
    until
    drop
;
: extract-uniqs ( songlen -- uniqlen ) \ 
     reset-uniqptr songmem swap 0 
     do \ loop over song grammar elements
        c@+ dup << < 
            if songuniq end?   \ found note
                if drop add-uniq \ reached end of list, so add
                else search-uniq
                then
            else drop \ ignore non-notes
            then 
    loop
    drop 
    uniqlen@
;
: play-uniqs ( uniqlen -- uniqlen )
  songuniq over 0 do c@+ note R2 loop drop ;
\ CODE REVIEW: need ORDER algorithm
: [uniqlights] ( uniqlen -- list uniqlen ) 
    >r songuniq r@ 0 do c@+ [light] swap loop drop r> ;
: uniqlights ( uniqlen -- ) [uniqlights] lightn ;
: songlights ( songlen -- songlen ) dup extract-uniqs uniqlights ;

\ TODO -- translate to light IDs and then light them!
\ ================= END UNDER CONSTRUCTION



\ Design: Takes song grammar from stack and writes notes to memory FIXED WRT TO A SCALE and INTERPRETING virtual octaves, adding rests grammar.  
\             Example: dream [writesong] play
: [writenote]     ( g -- )
        dup << >> between? 
        if   adjust-octave     \ CODE REVIEW: add light for adjusting octave
        else dup // /// between? 
              if  
              else [,,,] scaleroot@ + then
              songc!+  \ CODE REVIEW: count use of the array so know the length
        then 
;
: [writesong]     ( <notes> n -- songlen ) 
    reset-scaleroot init-songptr  reset-songlen  dup 0 
    do 
        dup i - roll 
        [writenote]
    loop drop songlen @ ;

\   Design: songlist puts the entire song notation onto the stack and then reads from stack left to right  CODE REVIEW: store into memory, read from memory
\   Design: core musical dependency is 3iSound-4x4.fs which has the notational transformation required
\   Design: audio only
: .. ( grammar -- )  \ notation is 3iSonglist grammar: 1..8 2% 2# >> << // ///  CX!!! in pace mode, extra delay because // pauses are read!
    dup // /// between?  if musical-rest
             else playmode-map playmode@ cells + @ execute 
             then ;
: flashlights  32-on 300 ms 32-clear 200 ms ;

\ Design: transposition is done from note index space to avoid having to translate back and forth to user's notation space
\ Design: back translation from transposition to notation space would give a different fingering
\ Design: only transpose notes (what's not between rests).  Transposition is 1-1, songlen unchanged
: c<!+ ( x ptr -- ptr ) 1 - c!+ ;
: [ntranspose] ( songlen xsteps -- ) transp ! songmem over 0 do c@+ dup // /// between? if else transp @ + then swap c<!+ loop drop ;

\ CODE REVIEW:  over-write to song if like it the transposition
\               how to get a fingered notation? i.e. a sensible scale
\               show all the notes of a scale... to see if one likes the fingering
\ TODO - store songs in EEPROM (non-volatile) for playing/replaying rapidly - may lead to timing issues
\ TODO - scan program and highlight all keys to expose the underlying scale based on keys requested
\ : transpose-test ( g -- )  \ shows 7 steps of transformation between notations.   5flats  1 6 5 7 -->  6 11 3 5
\    cr dup .  [,,,] dup . scaleroot@ + dup . 1 + dup . scaleroot@ - dup . 12 mod dup . chr2dia dup . drop ;


\ Design: play is from memory, not stack
\ CODEREVIEW 
: playmem    ( songlen -- songlen ) songmem over 0 do c@+ .. loop drop ;
: [play]     ( songlen -- ) [writesong] init-songptr songptr @ swap 0 do c@+ .. loop drop ;
: play       ( songlen -- ) soundonly-mode  [play] ;
: victory-jingle  80 setbeat  soundonly-mode  1 3 5 8 // 2 4 6 >> 2 <<    11 play 120 setbeat ;
: welldone!  flashlights flashlights   victory-jingle ;
: watch      ( songlen -- ) soundlight-mode       [play] ;
: pace       ( songlen -- ) soundlightkey-mode    [play] ; \ learns the pace to play it at, keyboard control
: learn      ( songlen -- ) soundlightbutton-mode [play]  welldone! ; 
: listenstep ( songlen -- ) soundkey-mode         [play] ;
: lightstep  ( songlen -- ) lightkey-mode         [play] ;
: [signature] ( songlen -- songlen ) dup extract-uniqs uniqlights ;
: signature   ( list songlen -- ) [writesong] [signature] ;
: transpose  ( songlen -- songlen ) 1 [ntranspose] [signature] ;
: ntranspose ( songlen xsteps -- songlen ) [ntranspose] [signature] ;

\ interesting: starting on E makes a lot of complicated songs easy!  dream -1   stairway 4
\ TODO: Add a "next note" button in case the learn mode gets stuck...

\   Design: audio + lights
\  : watch ( n -- ) dup 0 do dup i - roll dup #_ .. loop drop 32-clear ;   \ see 3iPianoLights.fs for kk
\  : watch ( n -- ) dup 0 do dup i - roll #__ 500 ms light| 100 ms loop drop light| ;   \ see 3iPort-4x4.fs for light_

\   Design: learn each note and use PC to prompt next note --- LATER: read key to check it's the right one being pressed
\ : learn ( n -- ) dup 0 do dup i - roll dup #_ .. key drop loop drop 32-clear ;
\ : learn ( n -- ) reset-scaleroot dup 0 do dup i - roll .. key drop loop drop 32-clear ;

\ TO DO: add ESC to play rest.  Add back key to go back.  Add play a phrase key.
\ TO DO: how to flash?


\ ================== PLAYLIST ==================
\ Design: each piece must end on the same octave that started, otherwise the LEARN mode will start off on the wrong octave after [writesong]

\ Classical Pieces
\ https://www.youtube.com/watch?v=VfPGNafN8K8
\ https://www.google.com/search?client=firefox-b-d&q=simple+classical+tunes

: new-harp  \ 0 [transpose]
    cr ." Y Delyn Newydd - The New Harp (Trad. Welsh)"
    cr ."    ... considered a Welsh Polka, with two others forming a trio: Pwt-ar-y-bys and Pant Corlan yr Wyn"
   130 setbeat 
   cstd
   3 2 1 2 3   5 5 4 4 ///
   4 3 2 3 4   6 6 5 5 ///
   3 2 1 2 3   5 5 4 4 ///
   4 3 2 /// 4 3 2 /// 3 1 1 ///
   8 7 6 5 8 7 6 5 8 7 6 5 4 3 2 ///
   7 6 5 /// 7 6 5 /// 7 6 5 4 3 2 1 ///
74
; 


\ Beethoven's Ode to Joy
: ode   \ 0 [transpose]
    cr ." Ode to Joy, by Ludwig van Beethoven, Symphony No. 9 in D Minor, Op 125 (1822)"
    cr ."    ... this was the final complete symphony by Beethoven 3 years before his death"
    cstd
    160 setbeat
    3 3 4 5 5 4 3 2 1 1 2 3 3 // 2 2 ///
    3 3 4 5 5 4 3 2 1 1 2 3 2 // 1 1 ///
    2 2 3 1 // 2 3 4 3 1 // 2 3 4 3 2 1 2 << 5 >> ///
    3 3 4 5 5 4 3 2 1 1 2 3 2 // 1 1   ///
73
;

\ Bach: Jesu, Joy of Man's Desiring  (Jesu, bleibet meine Freude) BWV 147/10, Last chorus of the cantata Herz und Mund und Tat und Leben BWV 147, 14:13
\ History and Evolution: https://artandtheology.org/2017/12/17/the-evolution-of-jesu-joy-of-mans-desiring/
\ https://www.youtube.com/watch?v=6ZKGD2IIhjk
\ Cello, Guitar Duo: https://www.youtube.com/watch?v=KialZHRdD7A
\ Piano (liquid legato): https://www.youtube.com/watch?v=uKq5IcBFyeQ
\ Organ: https://www.youtube.com/watch?v=2uXn6nDN7ck    
\ Jesus, joy of man's desiring 
\ Holy wisdom, love most bright 
\ Drawn by Thee, our souls aspiring Soar to uncreated light 
\ Word of God, our flesh that fashioned 
\ With the fire of life impassioned 
\ Striving still to Truth unknown Soaring, dying 'round Thy throne
: jesu-c  \ 0 [transpose]
  cr ." Jesu Joy of Man's Desiring, by Johann Sebastian Bach, 1723"
  cstd
  140 setbeat
  1 2 3 5 4 4 6 5 5 8 7 8 5 3 1 2 3 4 2 << 7  5 6 7 >>
  1 2 3 5 4 4 6 5 5 8 7 8 5 3 1 2 3 4 5 6 5 3 2 << 5 >> 1 << 7 >> 1
  56
;

: jesu-g  \ 0 [transpose]
  cstd
  140 setbeat
  5 6 7 >> 2 1 1 3 2 2 5 4# 5 2 << 7 5 6 7 8 6 5 2 5 4# 5
  26
;

: jesu-loop  begin jesu-c watch jesu-g watch key? until ;  \ BUG!!! Why is there a slight pause in between songs?  Is it stack load delay?

\ Fur Elise
\ https://www.sheetmusicdirect.com/en-US/se/ID_No/433968/Product.aspx
: elise  \ 0 [transpose]
  cstd
  140 setbeat
  >> 3 2# 3 2# 3 << 7 >> 2 1 << 6
  << 3 6  >> 1 3 6 7     3 5# 7 >> 1 //
  3 2# 3 2# 3 << 7 >> 2 1 << 6
  << 3 6  >> 1 3 6 7     3 >> 1 << 7 6 ///
  
  7 >> 1 2 3 << 3 >> 4 3 2 << 3 >> 3 2 1  << 3 >> 2 1 << 7 >> 3 3 3 3 

  3 2# 3 2# 3 << 7 >> 2 1 << 6
  << 3 6  >> 1 3 6 7     3 5# 7 >> 1  ///
  3 2# 3 2# 3 << 7 >> 2 1 << 6
  << 3 6  >> 1 3 6 7     3 >> 1 << 7 6 /// 
136
;


\ Pachelbel: Canon in D, 00:00 - 4:45
\ https://www.sheetmusicdirect.com/en-US/se/ID_No/103307/Product.aspx


\ Beethoven: Moonlight Sonata


\ FAMOUS POP TUNES


: memory  \ 0 [transpose]
    cr ." Memory (from the musical Cats), by Andrew Lloyd Weber, first sung by Elaine Page (1981), also covered by Barbara Streisand (1981)"
    cr ."    ... the song is sung by the cat Grizabella"
    160 setbeat
    cstd
    >> 8 // 8 //
    7 8 >> 2 << 8 6 8 // 8 //
    7 8 >> 2 << 8 5 6 // 6 //
    4 5 6 5 4 3 ///
    3 5 5 // 2      //
    3 4 5 6 7 8 7 6 5 ///
    3 // 1 5 ///
    << 6 6 >> // 1 1 /// << 
64
;

\ EXAMPLE of how use of scales allows converting a "difficult" piece requiring use of chromatic (13-note) scale into something simpler using a different diatonic (8-note) scale
: stairway-std
  cstd
  4 5 6% 5 4 5    //
  4 5 6% 7% 6% 5  // 
  6% 7% 8 7% 6% 5 4 3% 3% 4 4  ///
  4 >> 1# << 8 ///
32
;  


\ This is in the key of Eb major, starting at f (note 3)
create 3flats8 c_ e%_ f_ g_ a%_ b%_ c' c#' ( e%' ) write-key
: 3flats 3flats8 set-key ;

: stairway
  140 setbeat
  3flats \ slid down one note to hit the "alarmed now"
   
   3 // 4 // 5 // 4 3 4   // \ there's a lady ...
   3 4 5 6 5 4   // \  ... gold
   5 6 7 6 5 4 3 2 2 3 3  // \ ... to heaven
   3 8 7 ///

   3 4 5 4 3 4  // \ when she gets there ...
   3 4 5 6 5 4  // \ ... closed
   5 6 7 6 5 4 3 2 2 3 3  // \ ... what she came for
   3 8 7 ///

   5 // 7 6 // 6 5 4 5 3  \ oooo 
   3 4 5 6 7 6 5 4 3 2 2 3 3  /// \ and she's buying ... heaven

///

\ Verse 2   
   3 4 5 4 3 4   // \ there's a sign ...
   3 4 5 6 5 4   // \  ... sure
   5 6 7 6 5 4 3 2 2 3 3  // \ ... meanings
   3 8 7 ///

   3 4 5 4 3 4  // \ in a tree ...
   3 4 5 6 5 4  // \ ... sings
   5 6 7 6 5 4 3 2 2 3 3  // \ ... misgiven
   3 8 7 ///

   5 // 7 6 // 6 5 4 5 3  \ oooo 
   3 4 5 6 7 6 5 4 3 2 2 3 3  /// \ and she's buying ... heaven
  ///

   7 6 5 6 3 3 2 3 5  // \ if there's a rustle ...
   3 3 2 3 1  //   \ don't be alarmed now 
   5 5 7 6 5 4 // 2 5 2 3 /// \ it's just a spring clean for the may queen

   7 6 5 6 3 3 2 3 5 //  \ dear lady ...
   3 2 3 1 //  \ and do you know ...
   5 5 7 6 5 3 // 5 2 2 3 /// \ ... whispering wind   
   /// 
   
   5 // 7 6 // 6 5 4 5 3  \ oooo 
   3 4 5 6 7 6 5 4 3 2 2 3 3  /// \ and she's buying ... heaven
  ///

   5 /// 6 /// 7 /// //
   6 /// 5 /// 6 /// 5 /// 4 /// //
   2 /// 5 /// 2 // 3  /// \ and she's buying ... heaven
280
;

\ also works in the 5flats scale (D% major), starting with note 2
\ also works in cstd starting with note 2 D without any sharps or flats


\ abba I have a dream
: dream   \ or 1 flat (F major)  \ -1 [ntranpose] or 11 [ntranspose]
cr ." I have a dream, by Abba"
cr ." Switching to D% major (5-flats) scale" cr 500 ms
    5flats 
1 // 6 // 5 7 ///
1 // 4 // 3 4 ///
1 // 6 // 5 7 ///
5 // 4 3 4 ///
6 // 7 8 >> // 2 1 << 5 ///
5 // 6 7 // 8 6 ///
6 // 7 8 >> // 2 << 8 5 ///
5 // 6 7 // 8 6 ///
>> 1 // 2 3 // 4 3 << 7 ///
7 8 >>  2 3 2 // 1 << 7 // 6 8 8 ///
>> 1 // 2 3 // 4 3 << 7  /// 
7 8 >> 2 3 2 // 1 << 7 // 6 8 8 // 
1 // 6 // 5 7 ///
1 // 4 // 3 4 ///
131
; 

\  dream  watch
\  songlen @ 11 ntranpose


\ YU -- I see Fire by Ed Sheeran
: fire  \ 0 [transpose]
160 setbeat
yuG-approx
2 // 3 4 5 // 4 3 3 2 1 2 ///   \  ohhhh  misty eye ...
2 // 3 4 5 // 4 3 // 3 4 4 ///
2 // 3 4 5 7 // 6 5 4 // 5 4 3 4 3 ///
2 // 3 4 5 4 3 4 // 3 2 1 2 ///
57
;

: hallelujah  \ 0 [transpose]
cr ." Hallelujah, by Leonard Cohen"
cstd
  3 // 5 // 5 5 // 5 6 6 6 ///
  3 // 5 // 5 5 // 5 5 6 6 6 ///
  6 6 // 6 6 // 6 // 5 5 3 5 5 ///
  3 4 5 // 5 5 // 5 6 // 6 7 ///
  8 8 8 8 8 8 8 8 >> 2 ///
  2 2 2 // 2 // 2 3 3 3 2 2 1 << ///
  
  3 5 6 // 6 // 
  6 5 3 // 3 // 
  3 5 6 // 6 // 
  6 5 3 /// 4 3 2 /// 1 1 ///

  3 5 6 // 6 //
  6 5 3 // 3 // 
  3 5 6 // 6 // 
  6 5 >> 3 // 4 3 2 // 1 << ///
137
;

: wings \ 0 [transpose]
    cr ." Wind Beneath my Wings, Bette Midler (1988)"
    cr ."   ... Written 1982 by Jeff Silbar/Larry Henley."  
    cr ." Midler recorded it in 1988 and it went on to become #1 in US, #3 in UK, and sell over a million copies"
    cstd
    << 5 >> 3 // 4 5 4 3 2 1 1 // << 5 6 >> ///
    << 5 >> 3 // 4 5 4 3 2 1 1 ///
    3 4 3 2 1 1 3 2 //
    << 7 7 // 7 6 5 >> ///
    4 3 4 3 2 1 3 2 /// 

    << 5 >> 3 // 4 5 4 3 2 1 1 // << 5 6 >> //
    << 5 >> 3 // 4 5 4 3 2 1 1 ///
    4 3 4 3 2 1 3 2 /// 
    << 7 7 // 7 6 5 >> ///
    4 3 4 3 2 1 6 5 /// 
        
    1 // 6 7 8 7 6 5 4 5 // 3 ///
    1 // 6 7 8 7 6 5 4 5 // 
    1 // 6 7 8 7 6 5 4 5 // 3 2 1  //
    << 5 >> 2 3 4 3 2 1 << 7 >> 1 //
169 
; 

: room   \ 0 [transpose] or 3 [transpose] or 4 [transpose] or ** 5 [transpose]
cr ." In My Room, by The Beach Boys"
180 setbeat 
cstd
  2 2 3 2 4# 4# 5 4# 6 6 7 5 6 ///
  6 4# 3 /// 
  5 4# 2 ///
22
;
   
: green-day  \ 0 [transpose]
cstd
  5 7 8 7   5 7 8 7   5 7 8 7  5 // 5 5
16
;

: star-wars  \ 0 [transpose]
cstd
    1 // 5 // 4 3 2 8 /// 5 //  4 3 2 8 /// 5 // 4 3 4 2 ///
  1 1 // 5 // 4 3 2 8 /// 5 //  4 3 2 8 /// 5 // 4 3 4 2 ///
47
;

: top-gun  \ 0 [transpose]
cstd
  1 5 5 // 4 3 4 3 2 2 // 1 2 3 2 3 4 3 1 3 2 ///
1 1 5 5 // 4 3 4 3 2 2 // 1 2 3 2 3 4 3 1 5 ///
  1 2 3 4 5 6 7 >> //
  1 5 5 // 4 3 4 3 2 2 // 1 2 3 2 3 4 3 1 3 2 ///
1 1 5 5 // 4 3 4 3 2 2 // 1 2 3 2 3 4 3 1 5 ///
97
;

: rainbow   \ 0 [transpose]
    cr ." Somewhere over the Rainbow, by Izzy Kamakoele"
    1 // 8 // 7 5 6 7 8 //
    1 // 6 5 ///
    1 // 4 // 3 1 2 3 4 // 2 << 7 8 >> 2 3 1 ///
34
; 

: letitgo 
cr ." Let It Go, from Frozen, by Idina Menzel"
cstd
6 6 6 // 6 6 6 5 4 4 // 
4 4 5 5 // 5 4 2 ///
2 6 6 6 6 8 8 6 //
5 4 5 5 // 5 4 5 ///

6 6 8 >> 2 1 << 6 8 8 8 7% 6 7% 6 ///
6 5 4 5 // 5 6 5 4 2 ///

5 5 8 8   // 5 5 >> 2 2  ///
2 1 2 2 // 2 2 3 4 3 3 << /// 
5 5 8 8 //   5 5 >> 2 2 ///

2 2 3 4 ///
2 3 4 // 1 1 5 ///
4 5 6 6 // 7% 6 5 4 ///
8 6 5 // 4 4 8 8 6 7% 6 ///
6 6 5 1 8 ///
4 4 3 4 4 4 4 1 1 //
151
;

: reason \ 0 [transpose]
    cr ." Just Give Me a Reason, by Pink and Nate F."
    cstd
    6 6 6 6 // 4 5 6 7% 7% 7% 7% 4 ///  \ Right from the start ...
    4 >> 2 /// 2 1 // << 4 6 6  ///
    4 6 6 6 6 6 4 5 6 7% 7% 7% 4 4 ///  \  I let you see...
    4 4 >> 2 /// 2 1 // << 4 6 6 ///
    
    4 >> 4 4 4 3 2 // 5 // 3 // 1 //  4 // 1  ///  \ Now you've been having ...
         4 4 4 3 2 // 5 // 3 // 1 //  4 // 1  ///
       1 4 4 4 3 2 // 5 // 3 // 1 //  4 // 1 // << 7% /// 6 /// 5 /// 6 /// 7% ///
    
    >> 6 6 4 4 1 1 //  \ Just give me a reason ... 
    6 6 5 5 3 3 1 //
    5 5 4 4 4 4 4 3 2 3 // 
    1 1 1 7% 6 5 4 6 
    
    4 4 1 1 //   \  it's in the stars...
    6 6 5 5 3 3 1 // 
    5 5 4 4 4 4 4 3 2 3 // 
    1 1 1 7% 6 5 4 4 // 
    << 
185
;

\ TRADITIONAL

: daffodils  \ 0 transpose or 2 ntranspose] or ** 7 ntranspose
    cr ." I Love the Daffodils (English Trad.)"
    cr ."    ... This can be sung as a round, with the second singer starting on the 3rd phrase"
160 setbeat
    cstd
    4 4 3 2 3  // 
    5 5 4 3 4 5 // 
    6 6 5 4 6 // 
    7% 7% 6 5 6 7% // 
    8 8 7% 6 4 7% 6 5 4 3 // 
    1 2 3 4 4 3 2 2 2 5 5 4 3 // 
    1 2 3 4 4 3 2 2 2 5 5 4 3 ///
65
;

: scarborough  \ 2 ntranspose
  cr ." Scarborough Fair, English Traditional"
  160 setbeat
  cstd
  1 /// 1 // 5 5 // 5 2 3% 2 1 ///
  5 7% 8 // 7% 5 6 4 5 ///
  8 8 // 8 7% // 5 5 4 3% 2 1 ///
  1 // 5 4 // 3% 2 1 << 7% >> 1 ///
49
;  

: lavender  \ 0 ntranspose
cr ." lavender blue"
170 setbeat
cstd
  1 5 5 5 // 4 3 2 1  // 1 6 6 6 ///
  1 5 5 5 // 4 3 2 1  // 4 3 2 1 ///
  
  1 5 5 5 // 4 3 2 1  // 1 6 6 6 /// 
  1 5 5 5 // 4 3 2 1  // 4 3 2 1 ///
60
;

: cockles \ 0 ntranspose
cr ." cockles and mussels"
170 setbeat
cstd
    << 5 >> // 1 1 // 1 // 1 3 /// 1 1 // 2 /// 2 2 // 2 4 ///
    2 // 3 3 3 5 4 3 //  3 2 // 1 2  ///
    << 5 5 >> 1 // 1 1 // 1 3  /// 1 // 2 // 2 2 // 2 4 ///
    3 1 5 3 // 1  // 5 3 //
    1 // 2 // 3 2 // 1 ///
79
;

\ ETHNOMUSIC

: sea  \ 2 ntranspose
\ try irish key
 cr ." Song of the Sea, by Lisa Hannigan"  \ https://en.wikipedia.org/wiki/Selkie
 cr ." From sountrack of the Irish animated film (2014) about a Selkie child and her brother."
  170 setbeat
  cstd
     1 /// 1 5 /// 5 6 4 5 ///
     1 /// 1 5 /// 5 6 4 // 8 7% 6 5 ///
  >> 1 /// 1 /// 2 // 3% // 4 3% 2 ///
     1 /// 1 // 2 // 3% // 4 3% 5 << ///
     1 /// 1 5  // 5 6 4 5 ///
     1 /// 1 5  // 5 6 4 // 8 7% 6 5 ///
73
;



: morning  \ 0 ntranspose
    cr ." Morning Has Broken, by Cat Stevens"
170 setbeat
cstd
    1 3 5  >> 1 /// 2 << ///
    7 // 6 5 // 6 // 7 6 5 /// 
    1 3 5 // 5 /// 6 ///
    5 // 3 1 2 ///

    5 3 5 8 /// 6 ///
    5 3 1 1 // << 6 >> 1 2 // 
    
    3 2 3 5 /// 6 /// 
    2 3 // 2 1 /// 
69
;


\  PERSONAL COMPOSITIONS
\ Assad Ebrahim - Composition - 2020-06-28---Sun---21:53
: AE1 
    6 >> 1 3 << 6 >> 1 3 << 6 >> 1 3 <<
    5 7 >> 2 << 5 7 >> 2 << 5 7 >> 2 <<   5 >> 3 2 1 
    3 5 6   3 8 7   3 8 7    3 8 7 6 << ;
: AE2
    6 >> 1 3 <<  6 >> 1 3 <<  6 >> 1 3 << 
    >> 3 8 7  3 8 7   3 8 7 6 <<
    6 >> 1 3 <<  6 >> 1 3 <<  6 >> 1 3 << 
    >> 8 7 5  8 7 5  8 7 5 6 <<  ;
: AE3 
    6 >> 1 3 <<  6 >> 1 3   8 7 5  8 7 5 6 << ;  \ removing the last << takes it to the next octave - cool!
    
: yearning  
cr ." Yearning, by Assad Ebrahim"
170 setbeat
cstd
ae2 ae3 ae1 ae3 
137 ;


: dreaming  \  by Meera Shah Ebrahim 2021-05-16---Sun---21:34
cr ." Dreaming, by Meera Shah Ebrahim"
170 setbeat
cstd
>> 1 2 3 3  1 2 3 3  1 2 3 2 1 << 7 
7 8 >> 2 2  << 7 8 >> 2 2  7 8 >> 2 3 2 1
3 2 1 1 << 3 2 1 1  3 2 1 2 1 << 7 
7 >> 1 2 2  << 5 7 >> 2 2 << 5 7 >> 2 3 2 1
69
;

: together   \ Together, by Jasmine Sheherazade Ebrahim
cr ." Together, by Jasmine Sheherazade Ebrahim"
cstd
8  7  8 7 8 7 8 6 6 ///
7  6  7 6 7 6 7 5 5 ///
6  5  6 5 6 5 6 4 4 ///
5  4  3 8 8 3 4 ///
38
;


: snake  \ Snake Charmer, by Assad Ebrahim
cr ." Snake Charmer, by Assad Ebrahim"
cstd
 1  // 3  4 5 ///
 5  6% 5 6% 5 6% 4  // 5 ///
 1  // 3 4 5 ///
 3  4 // 3 4 // 3 4 3 // 1 ///
   
 1  // 3 4  5 ///
 5  6% 7% 6% 5 4 5 ///
 4  3 4 3 4 3 1 ///
 
 7% 6%  7% 6%  7% 6%  5 4  5 ///
 1 // 3 4 5 ///
 3 4 3 4 3 4 3 // 1 ///
82
;


: planets  \  Planets, by Assad Ebrahim - 2020-07-10---Fri---03:32
cr ." Planets, by Assad Ebrahim"
cstd
 3 5 6 
 3 5 6 // 8 7 6 5 6 ///
 13
;


: jimmy-boy  \ Jimmy Boy, by Assad Ebrahim  2020-07-15---Wed---03:38
cr ." Jimmy Boy, by Assad Ebrahim"
 cmaj
 1 2 3 5    5 6 5 4 5       \ oh jimmy-boy!  why are you running?
 1 2 3 5    5 4 3 3 4       \ oh jimmy-boy!  what have you done?
 1 2 3 5    5 6 5 4 5       \ oh jimmy-boy!  where will you run to?
 1 2 3 4    4 5 4 3 2 3     \ oh jimmy-boy!  I fear your time has come.
37 
;

\ Assad Ebrahim - 2021-05-12---Wed---05:18
: falling
200 setbeat
cstd
cr ." Falling Down, by Assad Ebrahim"
\ this uses the pentatonic scale  5_ 7_ 1 2 4 5    g b c d f 
  2 5 4 2 1 2 ///       \ the walls came tumbling down
  << 8 7 8 7 8 7 5 /// \ falling, falling, falling down 
\ shifting to start on g
  >> 5 8 7 5 4 5 ///
   4 3 4 3 4 3 1 ///
32
;


\ CHILDRENS

: twinkle
200 setbeat
cstd
    1 1 5 5 6 6 5 ///
    4 4 3 3 2 2 1 ///
    5 5 4 4 3 3 2 ///
    5 5 4 4 3 3 2 ///
    1 1 5 5 6 6 5 ///
    4 4 3 3 2 2 1 ///
    48
;


: mary
cr ." mary had a little lamb"
cstd
    3 2 1 2 3 3 3 //
    2 2 2 //
    3 5 5 ///
    3 2 1 2 3 3 3 // 1 2 2 3 2 1 ///
    31 ;

: doh
cr ." Doh a deer, Sound of Music"
cstd
    1 // 2 3 // 1 3 1 3 ///
    2 // 3 4 4 3 2 4 ///
    3 // 4 5 // 3 5 3 5 ///
    4 // 5 6 6 5 4 6 ///
    5 // 1 2 3 4 5 6 ///
    6 // 2 3 4 5 6 7 ///
    7 // 3 4 5 6 7 8 ///
    8 7 6 4 7 5 8 //
    1 2 3 4 5 6 7 8 ///
82
;

: alouette 
cstd    
    1 2 3 3 //
    2 1 2 3 1 << 5 >> ///
    1 2 3 3 //
    2 1 2 3 1 ///
    1 2 3 4 5 5 5 //
    5 6 5 4 3 2 1 ///
    5 5 5 ///
    1 1 1 ///
    5 5 5 ///
    1 1 1 ///
    5 4 3 2  // \ repeat
62
;


: row-row
cstd
    1 // 1 // 1 2 3 //
    3 2 3 4 5 ///
    8 8 // 5 5 // 3 3 // 1 1 //
    5 4 3 2 1 ///
32
; 

: london-bridge
cstd
    5 6 5 4 3 4 5 //
    2 3 4 // 3 4 5 ///
    5 6 5 4 3 4 5 //
    2 // 5 // 3 1 ///
31
;

\ jack and jill

: weasel  \ pop goes the weasel?
cstd
    1 // 1 2 // 2 3 5 3 1 ///
    << 5 >> 1 1 // 2 4 3 // 1 ///
    << 5 >> 1 1 // 2 2 // 3 5 3 1 /// 
    6 // 2 4 3 // 1 ///
    8 8 6 8 7 7 5 //
    5 8 8 6 8 7 7 5 //
    4 3 4 5 6 7 8 //
    8 // 2 4 3 1 ///
77
;
    
\ little bo peep


\ baa baa black sheep
: baa
cstd
  1 1 // 5 5 // 6 7 8 6 5 ///
  4 4 // 3 3 // 2 2 // 1 ///
  5 5 5 4 4 4 3 3 3 2 ///
  1 5 5 5 4 5 6 4 3 2 2 1 ///
47
;


: frere 
cr ." frere jacques"
cstd
     1 2 3 1 //  1 2 3 1  //
     3 4 5   /// 3 4 5    ///
     5 6 5 4 3 1 //  5 6 5 4 3 1 //
     1 << 5 8 ///  8 5 8 >> ///
42
;


: robin-hood
160 setbeat
cstd
    2 // 2 // 2 // 3 4 5 // 5 // 5 //
    6 7 8 // 5 // 5 6 5 4 3 // 1 //
    5 // 1 1 1 3 4 5 5 5 //
    6 7 8 // 5 // 5 6 5 4 3 1 //
    5 6 // 6 // 6 // 5 4 3 4 5 6 5 //
    6 7 8 // 5 // 5 6 5 4 3 /// 1 //
81
;

\ bonnie lies over the ocean
: bonnie
180 setbeat
cstd
    1 6 5 4 5 4 2 1 << 6 >> ///
    1 6 5 4 4 3 4 5   ///
    1 6 5 4 5 4 2 1 << 6 >> ///
    1 2 5 4 3 2 3 4  ///
    
    1 /// 4 // 3 2 /// 5 // 4 3 3 3 3 2 3 4 // 5 6 ///
    1 /// 4 // 3 2 /// 5 // 4 3 3 3 3 2 3 4 ///
81
;

: bday
160 setbeat
cstd
    1 1 2 1 4 3 ///
    1 1 2 1 5 4 ///
    1 1 8 6 4 3 2 ///
    7% 7% 6 4 5 4 ///
29
;


\ old macdonald



: uk-anthem  \ 7 ntranpose
cr ." My Country Tis and UK National Anthem God Save Our Queen"
220 setbeat
cstd
    4 4 5 3 // 4 5 //
    6 6 7% 6 // 5 4 //
    5 4 3 4 // 
    4 5 6 7% //
    8 8 8 8 // 7% 6 //
    7% 7% 7% 7% // 6 5
    6 // 7% 6 5 4 6 // 7% 8 
    >> 2 << 7% 6 // 5 // 4 ///
61
; 

\ star spangled banner
: USA \ -5 ntranspose
cstd
    8 6 4 // 6 // 8 // >> 4 ///
    6 // 5 4 << //
    6 7% 8 ///
    8 8 >> 6 // 5 4 3 ///
    2 3 4 // 4 // 1 // << 6 // 4 ///
    8 6 4 // 6 8 >> 4 ///
    6 5 4 // << 6 7% 8 ///
    8 8 >> 6 /// 5 4 // 3 ///
    2 3 4 // 4 // 1 // << 6 // 4 ///
    >> 6 6 6 // 7% 8 8 /// 
    7% 6 5 // 6 7% 7% ///
    7% // 6 // 5 4 3 ///
    2 3 4 << 6 7% 8 ///
    8 // >> 4 // 4 // 4 3 2 // 2 // 2  // 5 6 7% 6 5 4 4 /// 3 ///
    1 // 1 4 // 5 6 7% 8 ///
    4 5 6 // 7% // 5 // 4 /// <<
163
;     

\ Kenyan Anthem

\ AMERICAN FOLK SONGS

\ swing low sweet chariot
: chariot
cstd
    6 4 /// 6 4 // 4 2 1 ///
    4 4 4 5 6 6 8 // 8 ///
    >> 2 << 8 6 /// 8 4 // 4 2 1 ///
    4 4 4 5 6 6 5 // 4 ///
43
;

\ yankee doodle

\ michael finnegan
: finnegan
cstd
    1 // 4 4 4 4 // 6 8 8 // 6 5 4 ///
    5 5 5 5 4 5 // 3 2 1 ///
    4 // 4 4 4 // 6 8 8 // 6 5 4 ///
    5 5 1 1 4 4 4  // 1 4 4 4 ///
53
;

: home  \ 7 ntranspose
\ or 1 flat in the scale of F major
cr ." Home on the Range"
cstd
  1 1 4 5 6 //  \ oh give me a home ... 
  4 3 2 // 7% 7% 7% //
  6 7% 8 // 4 4 4 3 4 5  ///
  1 1 1 4 5 6 //
  4 3 2 // 7% 7% 7% ///
  7% 7% 6 // 5 4 3 // 4 5 4 ///
  
  8 // 7% 6 // 5 6 ///
  1 1 // 4 // 4 4 4 3 4 5 ///
  1 1 1 4 5 6 //
  4 3 2 // 7% 7% 7% ///
  7% 7% 6 // 5 4 3 4 5 4 ///
98
;

\ america the beautiful

    
\ drunken sailor


: susanna  \ 1 ntranspose
cr ." O Susanna!"
120 setbeat
cstd
    4# 5# 6# >> // 1# 1# // 2# 1# << 6# 4# 5# 6# 6# 5# 4# 5# ///
    4# 5# 6# >> // 1# 1# // 2# 1# << 6# 4# 5# 6# 6# 5# 5# 4# ///
40
;    

\ she'll be coming round the mountain
: mountain \ 1 ntranspose
120 setbeat
cstd
    1# // 2# //   4# 4# 4# 4#  2# 1# 1# 2# 4# ///
    4#  5# //   6# 6# 6# 6# 8# 6# 5# 4# 5# ///
    8# 8# // 6# 6# 6# 6# 5# 4# /// 
    2#  2#  2# 2# 6# 6# ///
    6#  6#  6# 6# 5# 5# 6# 5# 4# ///
54
;

\ naussau


: cowboy  \ 2 ntranpose
180 setbeat
cr ." Cowboys Lament, folk song, covered by Johnny Cash"
\ https://en.wikipedia.org/wiki/Bury_Me_Not_on_the_Lone_Prairie
cstd
\ alt: Chinese yu scale
  1 /// 3% 3% // 5 // 7% ///  \ oh bury me not...
  8 7% 5 // 3% // 5 ///
  1 1 // 3% 5 4 ///
  5 4 3% // 1 3% ///
31
;  

: betsy  
160 setbeat
cr ." Sweet Betsy From Pike"
cstd
  1 1 3 5 5 4 2 2 1 << 7 >> 1 ///  \ oh do you remember ... 
  1 1 3 5 8 8 8 8 5 5 5 ///
  5 8 8 8 7 5 3 4 5 6 5 ///
  1 2 3 3 3 5 4 2 2 1 << 7 >> 1 ///
53
; 

: smokey   \ note - this is the basis for "Oh Where O Where Can My Baby Be"
180 setbeat
cr ." On Top of Old Smokey"
cstd
  1 1 3 5 8 ///  6 ///
  6 4 5 6 5 ///
  1 1 3 5 5 /// 2 ///
  3 4 3 2 1 ///

  1 1 3 5 8 /// 6 ///
  6 4 5 6 5 ///
  1 1 3 5 5 /// 2 ///
  3 4 3 2 1 ///
56
;


\ axle-f

\ chopstix

: silent-night  \ 2 ntranpose
180 setbeat
cstd
    4 // 5 4 2 ///
    4 // 5 4 2 ///
    7% // 7% 5 /// 
    6 // 6 4 ///
    5 // 5 7% // 6 5 // 4 // 5 4 // 2 ///
    5 // 5 7% // 6 5 // 4 // 5 4 // 2 ///
    >> 1 // 1 3% // 1 << 6 // 7% // >> 2 ///
    << 7% 4 2 4 // 3% 1 // << 7% /// >>
80
;

\ jingle bells


: fernando ;

: silence ;


: bridge ;

    
\ chords
: A.::  a_ c' e' ;


\ ======================================

: playlist  \ 55 songs
cr 
cr ."  PLAYLIST (55 tunes)"
cr ."  -------------------"
cr ."  new-harp   ode          jesu-c      jesu-g      jesu-loop    elise"
cr ."  memory     stairway     dream       fire        hallelujah   wings"
cr ."  room       green-day    star-wars   top-gun     rainbow      letitgo"
cr ."  reason"
cr ."  daffodils  scarborough  lavender    cockles     sea          morning"
cr ."  yearning   dreaming     tog         snake       planets      jimmy-boy"
cr ."  falling"
cr ."  twinkle    mary         doh         alouette    row-row      london-bridge"
cr ."  weasel     baa          frere       robin-hood  bonnie       bday"
cr ."  uk-anthem  usa"
cr ."  chariot    finnegan     home        susanna     mountain     cowboy"
cr ."  betsy      smokey       silent-night"
cr
;

: help-songlist
cr 
cr ." HELP-SONGLIST"
cr ." -------------"
cr ." by Assad Ebrahim - http://www.mathscitech.org/articles"
cr 
cr ." include 3iSonglist.fs  [3iForth.fs 3iSound-4x4.fs 3iShift.fs 3iPianoLights.fs]"
cr ." init: 1 open-com  13 start-speaker init-shiftout flashlights 1 buttonsrecord"
cr ." playlist"
cr ."      play | watch | pace | learn | signature | transpose | ntranspose "
cr 
cr ." Tools: "
cr ."    playmem    play from songmem, e.g. after transpose or ntranspose"
cr ."    ><<   >>>  octave down/up"
cr ."    help-scales   set white keys to diatonic scale for ethnomusic"
cr 
;

playlist
help-songlist
help-scales

\ V3 - 2021-05-16---Sun---17:14 - perfected the multi-octave notation with << and >> mapping to [<<] and [>>]
\ V4 - 2021-08-11---Wed---06:28 - MAJOR - perfected integration with 32-key piano lights board and different key signatures (ethnoscales), fixed songlist
\ V5 - 2021-08-20---Fri---07:04 - MAJOR - FEATURE (3iSonglist): Added true teach-and-learn capability to the piano! using poll-note-data (3iArpeggiator) to fetch data
\ V6 - 2021-08-21---Sat---08:30 - MAJOR - PARTIAL FEATURE: 
\ IN PROGRESS: partial transpose capability - runs into trouble juggling handling octave.
\    To Reproduce bug from I have a dream
\    5flats
\    6 7 8 16 2 1 15 5     8 play
\    6 7 8 16 2 1 15 5     8 0 ntranspose-song
\    cstd
\    4 11 12 15 13 16 15 12 16 10   10 play
\ V7 2021-08-24---Tue---01:36 MAJOR REVISION: play/watch/pace/learn first [writesong] to mem then play from mem using notes
\  Song grammer is parsed to write once in the note (fixing scale and octave adjustments)
\  After than transpositions are easy in note format from memory
\ V8 2021-08-25---Wed---05:33 -- fixed learning mode and corrected playlist -- 55 songs
\ V9 2021-08-27---Fri---05:07 -- added extract-uniq algorithm, features: show key-signature (light up all keys needed to play), transpose by 1 semitone or n semitones, annotated all 55 songs with optimal transposition to stay entirely on white keys
\ TODO - 2021-08-27---Fri---05:21 software is now good enough to design hardware user interface and test it out
